﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;

namespace HeuristicLab.Analysis.AlgorithmBehavior.Analyzers {
  public static class DistanceMatrixToPoints {
    /*
     *  Calculates a matrix of n-dimensional points from the distance matrix dm as described in 
     *  http://math.stackexchange.com/questions/156161/finding-the-coordinates-of-points-from-distance-matrix/423898#423898
     *  and
     *  http://stackoverflow.com/questions/10963054/finding-the-coordinates-of-points-from-distance-matrix/17177833#17177833
     *   
     */
    public static double[][] ConvertDistanceMatrixToPoints(double[][] dm) {
      double[][] points = new double[dm.Length][];
      for (int i = 0; i < dm.Length; i++) {
        points[i] = new double[dm.Length];
      }
      double[,] m = new double[dm.Length, dm.Length];
      double[] q = new double[dm.Length]; //eigenvalues
      double[,] v = new double[dm.Length, dm.Length];  //eigenvectors

      for (int i = 0; i < dm.Length; i++) {
        for (int j = 0; j < dm.Length; j++) {
          m[i, j] = 0.5 * (Math.Pow(dm[0][j], 2) + Math.Pow(dm[i][0], 2) - Math.Pow(dm[i][j], 2));
        }
      }

      bool res = alglib.smatrixevd(m, dm.Length, 1, true, out q, out v);
      if (!res) throw new Exception("Eigenvalue computation did not converge!");

      for (int i = 0; i < dm.Length; i++) {
        for (int j = 0; j < dm.Length; j++) {
          points[j][i] = Math.Sqrt(q[i]) * v[j, i];
        }
      }

      return points;
    }
  }
}
